import typing

import bson
import gridfs
import pymongo.client_session
import pymongo.collation
import pymongo.command_cursor
import pymongo.read_concern
import pymongo.read_preferences
import pymongo.write_concern

from . import core, motor_gridfs

_Collation = typing.Union[typing.Mapping[str, typing.Any], pymongo.collation.Collation]
_Document = typing.Dict[str, typing.Any]
_Pipeline = typing.Sequence[typing.Mapping[str, typing.Any]]
_Session = typing.Union[pymongo.client_session.ClientSession, AsyncIOMotorClientSession]
_ReadPreferences = typing.Union[
    pymongo.read_preferences.Primary,
    pymongo.read_preferences.PrimaryPreferred,
    pymongo.read_preferences.Secondary,
    pymongo.read_preferences.SecondaryPreferred,
    pymongo.read_preferences.Nearest,
]

def create_asyncio_class(cls: typing.Type) -> typing.Type: ...

class AsyncIOMotorClient(core.AgnosticClient):
    def __getitem__(self, item: str) -> AsyncIOMotorDatabase: ...
    def __getattr__(self, item: str) -> AsyncIOMotorDatabase: ...
    def get_database(
        self,
        name: typing.Optional[str],
        codec_options: typing.Optional[bson.codec_options.CodecOptions],
        read_preferences: typing.Optional[_ReadPreferences],
        write_concern: typing.Optional[pymongo.write_concern.WriteConcern],
        read_concern: typing.Optional[pymongo.read_concern.ReadConcern],
    ) -> AsyncIOMotorDatabase: ...
    def get_default_database(
        self,
        default: typing.Optional[str],
        codec_options: typing.Optional[bson.codec_options.CodecOptions],
        read_preferences: typing.Optional[_ReadPreferences],
        write_concern: typing.Optional[pymongo.write_concern.WriteConcern],
        read_concern: typing.Optional[pymongo.read_concern.ReadConcern],
        comment: typing.Optional[typing.Any],
    ) -> AsyncIOMotorDatabase: ...
    async def list_databases(
        self,
        session: typing.Optional[_Session],
        comment: typing.Optional[typing.Any],
        **kwargs: typing.Any,
    ) -> AsyncIOMotorCommandCursor: ...
    async def start_session(
        self,
        casual_consistency: typing.Optional[bool],
        default_transaction_options: typing.Optional[
            pymongo.client_session.TransactionOptions
        ] = None,
    ) -> AsyncIOMotorClientSession: ...
    def watch(
        self,
        pipeline: typing.Optional[
            typing.Sequence[typing.Mapping[str, typing.Any]]
        ] = None,
        full_document: typing.Optional[str] = None,
        resume_after: typing.Optional[typing.Mapping[str, typing.Any]] = None,
        max_await_time_ms: typing.Optional[int] = None,
        batch_size: typing.Optional[int] = None,
        collation: typing.Optional[
            typing.Union[typing.Mapping[str, typing.Any], pymongo.collation.Collation]
        ] = None,
        start_at_operation_time: typing.Optional[bson.timestamp.Timestamp] = None,
        session: typing.Optional[_Session] = None,
        start_after: typing.Optional[typing.Mapping[str, typing.Any]] = None,
        comment: typing.Optional[typing.Any] = None,
        full_document_before_change: typing.Optional[str] = None,
    ) -> AsyncIOMotorChangeStream: ...

class AsyncIOMotorClientSession(core.AgnosticClientSession):
    client: AsyncIOMotorClient

class AsyncIOMotorDatabase(core.AgnosticDatabase):
    client: AsyncIOMotorClient
    def __getitem__(self, item: str) -> AsyncIOMotorCollection: ...
    def __getattr__(self, item: str) -> AsyncIOMotorCollection: ...
    def __init__(self, client: AsyncIOMotorClient, name: str, **kwargs: typing.Any): ...
    def aggregate(
        self, pipeline: _Pipeline, *args: typing.Any, **kwargs: typing.Any
    ) -> AsyncIOMotorLatentCommandCursor: ...
    async def create_collection(
        self,
        name: str,
        codec_options: typing.Optional[bson.codec_options.CodecOptions] = None,
        read_preference: typing.Optional[_ReadPreferences] = None,
        write_concern: typing.Optional[pymongo.write_concern.WriteConcern] = None,
        read_concern: typing.Optional[pymongo.read_concern.ReadConcern] = None,
        session: typing.Optional[_Session] = None,
        check_exists: typing.Optional[bool] = True,
        **kwargs: typing.Any,
    ) -> AsyncIOMotorCollection: ...
    def get_collection(
        self,
        name: str,
        codec_options: typing.Optional[bson.codec_options.CodecOptions] = None,
        read_preference: typing.Optional[_ReadPreferences] = None,
        write_concern: typing.Optional[pymongo.write_concern.WriteConcern] = None,
        read_concern: typing.Optional[pymongo.read_concern.ReadConcern] = None,
    ) -> AsyncIOMotorCollection: ...
    async def list_collections(
        self,
        session: typing.Optional[_Session] = None,
        filter: typing.Optional[typing.Mapping[str, typing.Any]] = None,
        comment: typing.Optional[typing.Any] = None,
        **kwargs: typing.Any,
    ) -> AsyncIOMotorCommandCursor: ...
    def watch(
        self,
        pipeline: typing.Optional[_Pipeline] = None,
        full_document: typing.Optional[str] = None,
        resume_after: typing.Optional[typing.Mapping[str, typing.Any]] = None,
        max_await_time_ms: typing.Optional[int] = None,
        batch_size: typing.Optional[int] = None,
        collation: typing.Optional[_Collation] = None,
        start_at_operation_time: typing.Optional[bson.timestamp.Timestamp] = None,
        session: typing.Optional[_Session] = None,
        start_after: typing.Optional[typing.Mapping[str, typing.Any]] = None,
        comment: typing.Optional[typing.Any] = None,
        full_document_before_change: typing.Optional[str] = None,
    ) -> AsyncIOMotorChangeStream: ...

class AsyncIOMotorCollection(core.AgnosticCollection):
    def __init__(
        self,
        database: AsyncIOMotorDatabase,
        name: str,
        codec_options: typing.Optional[bson.codec_options.CodecOptions] = None,
        read_preference: typing.Optional[_ReadPreferences] = None,
        write_concern: typing.Optional[pymongo.write_concern.WriteConcern] = None,
        read_concern: typing.Optional[pymongo.read_concern.ReadConcern] = None,
        _delegate: typing.Optional[pymongo.collection.Collection] = None,
    ): ...
    def aggregate(
        self,
        pipeline: _Pipeline,
        *args: typing.Any,
        **kwargs: typing.Any,
    ) -> AsyncIOMotorLatentCommandCursor: ...
    def aggregate_raw_batches(
        self,
        pipeline: _Pipeline,
        **kwargs: typing.Any,
    ) -> AsyncIOMotorLatentCommandCursor: ...
    def find(
        self,
        filter: typing.Optional[typing.Mapping[str, typing.Any]] = None,
        projection: typing.Optional[
            typing.Mapping[str, typing.Any] | typing.Iterable[str]
        ] = None,
        skip: int = 0,
        limit: int = 0,
        no_cursor_timeout: bool = False,
        cursor_type: int = pymongo.cursor.CursorType.NON_TAILABLE,
        sort: typing.Optional[
            typing.Sequence[tuple[str, int | str | typing.Mapping[str, typing.Any]]]
        ] = None,
        allow_partial_results: bool = False,
        oplog_replay: bool = False,
        batch_size: int = 0,
        collation: typing.Optional[typing.Mapping[str, typing.Any] | _Collation] = None,
        hint: typing.Optional[
            str
            | typing.Sequence[tuple[str, int | str | typing.Mapping[str, typing.Any]]]
        ] = None,
        max_scan: typing.Optional[int] = None,
        max_time_ms: typing.Optional[int] = None,
        max: typing.Optional[
            typing.Sequence[tuple[str, int | str | typing.Mapping[str, typing.Any]]]
        ] = None,
        min: typing.Optional[
            typing.Sequence[tuple[str, int | str | typing.Mapping[str, typing.Any]]]
        ] = None,
        return_key: typing.Optional[bool] = None,
        show_record_id: typing.Optional[bool] = None,
        snapshot: typing.Optional[bool] = None,
        comment: typing.Optional[typing.Any] = None,
        session: typing.Optional[_Session] = None,
        allow_disk_use: typing.Optional[bool] = None,
        let: typing.Optional[bool] = None,
    ) -> AsyncIOMotorCursor: ...
    def list_indexes(
        self,
        session: typing.Optional[_Session] = None,
        comment: typing.Optional[typing.Any] = None,
    ) -> AsyncIOMotorLatentCommandCursor: ...
    def watch(
        self,
        pipeline: typing.Optional[_Pipeline] = None,
        full_document: typing.Optional[str] = None,
        resume_after: typing.Optional[typing.Mapping[str, typing.Any]] = None,
        max_await_time_ms: typing.Optional[int] = None,
        batch_size: typing.Optional[int] = None,
        collation: typing.Optional[_Collation] = None,
        start_at_operation_time: typing.Optional[bson.timestamp.Timestamp] = None,
        session: typing.Optional[_Session] = None,
        start_after: typing.Optional[typing.Mapping[str, typing.Any]] = None,
        comment: typing.Optional[typing.Any] = None,
        full_document_before_change: typing.Optional[str] = None,
    ) -> AsyncIOMotorChangeStream: ...

class AsyncIOMotorCursor(core.AgnosticCursor):
    session: AsyncIOMotorClientSession
    def __init__(
        self, cursor: pymongo.cursor.Cursor, collection: AsyncIOMotorCollection
    ) -> None: ...

class AsyncIOMotorCommandCursor(core.AgnosticCommandCursor):
    session: AsyncIOMotorClientSession
    def __init__(
        self, cursor: pymongo.cursor.Cursor, collection: AsyncIOMotorCollection
    ) -> None: ...

class AsyncIOMotorLatentCommandCursor(core.AgnosticLatentCommandCursor):
    session: AsyncIOMotorClientSession

    def __init__(
        self,
        collection: AsyncIOMotorCollection,
        start: typing.Optional[
            typing.Callable[
                [typing.Any], pymongo.command_cursor.CommandCursor[_Document]
            ]
        ],
        *args: typing.Any,
        **kwargs: typing.Any,
    ) -> None: ...

class AsyncIOMotorChangeStream(core.AgnosticChangeStream):
    def __init__(
        self,
        target: typing.Union[
            AsyncIOMotorClient,
            AsyncIOMotorDatabase,
            AsyncIOMotorCollection,
        ],
        pipeline: typing.Optional[
            typing.Sequence[typing.Mapping[str, typing.Any]]
        ] = None,
        full_document: typing.Optional[str] = None,
        resume_after: typing.Optional[typing.Mapping[str, typing.Any]] = None,
        max_await_time_ms: typing.Optional[int] = None,
        batch_size: typing.Optional[int] = None,
        collation: typing.Optional[
            typing.Union[typing.Mapping[str, typing.Any], pymongo.collation.Collation]
        ] = None,
        start_at_operation_time: typing.Optional[bson.timestamp.Timestamp] = None,
        session: typing.Optional[_Session] = None,
        start_after: typing.Optional[typing.Mapping[str, typing.Any]] = None,
        comment: typing.Optional[typing.Any] = None,
        full_document_before_change: typing.Optional[str] = None,
    ) -> None: ...

class AsyncIOMotorGridFSBucket(motor_gridfs.AgnosticGridFSBucket):
    collection: AsyncIOMotorCollection
    def __init__(
        self,
        database: AsyncIOMotorDatabase,
        bucket_name: str = 'fs',
        chunk_size_bytes: int = gridfs.DEFAULT_CHUNK_SIZE,
        write_concern: typing.Optional[pymongo.write_concern.WriteConcern] = None,
        read_preferences: typing.Optional[_ReadPreferences] = None,
        collection: typing.Optional[AsyncIOMotorCollection] = None,
    ) -> None: ...
    async def find(
        self,
        filter: typing.Optional[typing.Mapping[str, typing.Any]] = None,
        skip: int = 0,
        limit: int = 0,
        no_cursor_timeout: bool = False,
        sort: typing.Optional[typing.Any] = None,
        batch_size: int = 0,
        session: typing.Optional[_Session] = None,
    ) -> AsyncIOMotorGridOutCursor: ...
    async def open_download_stream(
        self, file_id: typing.Any, session: typing.Optional[_Session] = None
    ) -> AsyncIOMotorGridOut: ...
    async def open_download_stream_by_name(
        self,
        filename: str,
        revision: int = -1,
        session: typing.Optional[_Session] = None,
    ) -> AsyncIOMotorGridOut: ...
    def open_upload_stream(
        self, file_id: typing.Any, session: typing.Optional[_Session] = None
    ) -> AsyncIOMotorGridIn: ...
    def open_upload_stream_with_id(
        self,
        file_id: typing.Any,
        filename: str,
        chunk_size_bytes: typing.Optional[int] = None,
        metadata: typing.Optional[typing.Mapping[str, typing.Any]] = None,
        session: typing.Optional[_Session] = None,
    ) -> AsyncIOMotorGridIn: ...

class AsyncIOMotorGridIn(motor_gridfs.AgnosticGridIn):
    def __init__(
        self,
        root_collection: AsyncIOMotorCollection,
        delegate: gridfs.grid_file.GridIn,
        session: pymongo.client_session.ClientSession,
        **kwargs: typing.Any,
    ) -> None: ...

class AsyncIOMotorGridOut(motor_gridfs.AgnosticGridOut):
    def __init__(
        self,
        root_collection: AsyncIOMotorCollection,
        file_id: typing.Optional[int] = None,
        file_document: typing.Optional[typing.Any] = None,
        delegate: typing.Optional[gridfs.grid_file.GridOut] = None,
        session: typing.Optional[pymongo.client_session.ClientSession] = None,
        **kwargs: typing.Any,
    ) -> None: ...

class AsyncIOMotorGridOutCursor(motor_gridfs.AgnosticGridOutCursor):
    def next_object(self) -> typing.Optional[AsyncIOMotorGridOut]: ...

class AsyncIOMotorClientEncryption(core.AgnosticClientEncryption):
    def __init__(
        self,
        kms_providers: typing.Mapping[str, typing.Any],
        key_vault_namespace: str,
        key_vault_client: AsyncIOMotorClient,
        codec_options: bson.codec_options.CodecOptions,
        kms_tls_options: typing.Optional[typing.Mapping[str, typing.Any]] = None,
    ) -> None: ...
    async def get_keys(self) -> AsyncIOMotorCursor: ...



import datetime
from asyncio import AbstractEventLoop
from os import SEEK_SET as _SEEK_SET
from types import TracebackType
from typing import IO, Any, Iterable, List, Mapping, NoReturn, Optional, Type, Union

from bson import ObjectId
from gridfs.grid_file import DEFAULT_CHUNK_SIZE, GridIn, GridOut
from pymongo import read_preferences as _read_preferences
from pymongo.client_session import ClientSession
from pymongo.write_concern import WriteConcern
from typing_extensions import Literal, Self

from .core import (
    AgnosticClientSession,
    AgnosticCollection,
    AgnosticCursor,
    AgnosticDatabase,
)
from .metaprogramming import ReadOnlyProperty

_IO_Loop = AbstractEventLoop
_Handler = Any  # Replace it with tornado.web.RequestHandler
_ReadPreferences = Union[
    _read_preferences.Primary,
    _read_preferences.PrimaryPreferred,
    _read_preferences.Secondary,
    _read_preferences.SecondaryPreferred,
    _read_preferences.Nearest,
]
_Session = Union[ClientSession, AgnosticClientSession]

class MotorGridOutProperty(ReadOnlyProperty):
    pass

class AgnosticGridOutCursor(AgnosticCursor):
    def next_object(self) -> Optional[AgnosticGridOut]: ...

class AgnosticGridOut(object):
    _id: Any
    aliases: Optional[List[str]]
    chunk_size: int
    content_type: Optional[str]
    filename: Optional[str]
    length: int
    metadata: Optional[Mapping[str, Any]]
    name: Optional[str]
    upload_date: datetime.datetime

    def __init__(
        self,
        root_collection: AgnosticCollection,
        file_id: Optional[int] = None,
        file_document: Optional[Any] = None,
        delegate: Optional[GridOut] = None,
        session: Optional[ClientSession] = None,
        **kwargs: Any,
    ) -> None: ...
    async def __aiter__(self) -> Self: ...
    async def __anext__(self) -> bytes: ...
    def __getattr__(self, name: str) -> Any: ...
    async def _ensure_file(self) -> None: ...
    async def close(self) -> None: ...
    def get_io_loop(self) -> _IO_Loop: ...
    async def open(self) -> Self: ...
    def read(self, size: int = -1) -> bytes: ...
    def readable(self) -> Literal[True]: ...
    async def readchunk(self) -> bytes: ...
    def readline(self, size: int = -1) -> bytes: ...
    def seek(self, pos: int, whence: int = _SEEK_SET) -> int: ...
    def tell(self) -> int: ...
    def seekable(self) -> Literal[True]: ...
    async def write(self, data: Any) -> None: ...
    async def stream_to_handler(self, request_handler: _Handler) -> None: ...

class AgnosticGridIn(object):
    _id: Any
    chunk_size: int
    closed: bool
    content_type: Optional[str]
    filename: Optional[str]
    length: int
    md5: Optional[str]
    name: Optional[str]
    upload_date: datetime.datetime

    def __init__(
        self,
        root_collection: AgnosticCollection,
        delegate: Optional[GridIn] = None,
        session: Optional[ClientSession] = None,
        **kwargs: Any,
    ) -> None: ...
    async def __aenter__(self) -> Self: ...
    async def __aexit__(
        self,
        exc_type: Optional[Type[Exception]],
        exc_val: Optional[Exception],
        exc_tb: Optional[TracebackType],
    ) -> None: ...
    def __getattr__(self, name: str) -> Any: ...
    async def _exit(
        self,
        exc_type: Optional[Type[Exception]],
        exc_val: Optional[Exception],
        exc_tb: Optional[TracebackType],
    ) -> None: ...
    async def abort(self) -> None: ...
    async def close(self) -> None: ...
    def get_io_loop(self) -> _IO_Loop: ...
    def read(self, size: int = -1) -> NoReturn: ...
    def readable(self) -> Literal[False]: ...
    def seekable(self) -> Literal[False]: ...
    async def write(self, data: Any) -> None: ...
    async def writelines(self, sequence: Iterable[Any]) -> None: ...
    def writeable(self) -> Literal[True]: ...
    async def set(self, name: str, value: Any) -> None: ...

class AgnosticGridFSBucket(object):
    collection: AgnosticCollection
    def __init__(
        self,
        database: AgnosticDatabase,
        bucket_name: str = 'fs',
        chunk_size_bytes: int = DEFAULT_CHUNK_SIZE,
        write_concern: Optional[WriteConcern] = None,
        read_preferences: Optional[_ReadPreferences] = None,
        collection: Optional[AgnosticCollection] = None,
    ) -> None: ...
    async def delete(
        self, file_id: Any, session: Optional[_Session] = None
    ) -> None: ...
    async def download_to_stream(
        self,
        file_id: Any,
        destination: IO,
        session: Optional[_Session] = None,
    ) -> None: ...
    async def download_to_stream_by_name(
        self,
        filename: str,
        destination: IO,
        revision: int = -1,
        session: Optional[_Session] = None,
    ) -> None: ...
    async def find(
        self,
        filter: Optional[Mapping[str, Any]] = None,
        skip: int = 0,
        limit: int = 0,
        no_cursor_timeout: bool = False,
        sort: Optional[Any] = None,
        batch_size: int = 0,
        session: Optional[_Session] = None,
    ) -> AgnosticGridOutCursor: ...
    async def get_io_loop(self) -> _IO_Loop: ...
    async def open_download_stream(
        self, file_id: Any, session: Optional[_Session] = None
    ) -> AgnosticGridOut: ...
    async def open_download_stream_by_name(
        self, filename: str, revision: int = -1, session: Optional[_Session] = None
    ) -> AgnosticGridOut: ...
    def open_upload_stream(
        self, file_id: Any, session: Optional[_Session] = None
    ) -> AgnosticGridIn: ...
    def open_upload_stream_with_id(
        self,
        file_id: Any,
        filename: str,
        chunk_size_bytes: Optional[int] = None,
        metadata: Optional[Mapping[str, Any]] = None,
        session: Optional[_Session] = None,
    ) -> AgnosticGridIn: ...
    async def rename(
        self,
        file_id: Any,
        new_filename: str,
        session: Optional[_Session] = None,
    ) -> None: ...
    async def upload_from_stream(
        self,
        filename: str,
        source: Any,
        chunk_size_bytes: Optional[int] = None,
        metadata: Optional[Mapping[str, Any]] = None,
        session: Optional[_Session] = None,
    ) -> ObjectId: ...
    async def upload_from_stream_with_id(
        self,
        file_id: Any,
        filename: str,
        source: IO,
        chunk_size_bytes: Optional[int] = None,
        metadata: Optional[Mapping[str, Any]] = None,
        session: Optional[_Session] = None,
    ) -> None: ...



from types import ModuleType
from typing import (
    Any,
    Callable,
    Coroutine,
    Dict,
    Generator,
    Iterable,
    NoReturn,
    Optional,
    Tuple,
    Type,
    TypeVar,
)

from typing_extensions import ParamSpec, Self

_Func = TypeVar('_Func', bound=Callable)
_Args = ParamSpec('_Args')
_Ret = TypeVar('_Ret')
_KT = TypeVar('_KT')
_VT = TypeVar('_VT')

def asynchronize(
    framework: ModuleType,
    sync_method: Callable[_Args, _Ret],
    doc: Optional[str] = None,
    wrap_class: Optional[Type] = None,
    unwrap_class: Optional[str] = None,
) -> Callable[_Args, Coroutine[Any, Any, _Ret]]: ...
def unwrap_args_session(args: Iterable[Any]) -> Generator[Any, Any, None]: ...
def unwrap_kwargs_session(kwargs: Dict[_KT, _VT]) -> Dict[_KT, _VT]: ...
def coroutine_annotation(f: _Func) -> _Func: ...

class MotorAttributeFactory(object):
    def __init__(self, doc: Optional[str] = None): ...
    def create_attribute(self, cls: Type, attr_name: str) -> NoReturn: ...

class Async(MotorAttributeFactory):
    def __init__(self, attr_name: str, doc: Optional[str] = None) -> None: ...
    def create_attribute(
        self, cls: Type, attr_name: str
    ) -> Coroutine[Any, Any, Any]: ...
    def wrap(self, original_class: Type) -> Self: ...
    def unwrap(self, class_name: str) -> Self: ...

class AsyncRead(Async):
    attr_name: str
    wrap_class: Optional[Type] = None
    unwrap_class: Optional[str] = None
    def __init__(
        self, attr_name: Optional[str] = None, doc: Optional[str] = None
    ) -> None: ...

class AsyncWrite(Async):
    def __init__(
        self, attr_name: Optional[str] = None, doc: Optional[str] = None
    ) -> None: ...

class AsyncCommand(Async):
    def __init__(
        self, attr_name: Optional[str] = None, doc: Optional[str] = None
    ) -> None: ...

class ReadOnlyProperty(MotorAttributeFactory):
    def create_attribute(self, cls: Type, attr_name: str) -> property: ...

class DelegateMethod(ReadOnlyProperty):
    wrap_class: Optional[Type] = None
    def wrap(self, original_class: Type) -> Self: ...
    def create_attribute(
        self, cls: Type, attr_name: str
    ) -> Callable[[Any, Tuple[..., Any], Dict[str, Any]], Optional[Any]]: ...

class MotorCursorChainingMethod(MotorAttributeFactory):
    def create_attribute(
        self, cls: Type, attr_name: str
    ) -> Callable[[_Ret, Tuple[..., Any], Dict[str, Any]], _Ret]: ...

def create_class_with_framework(cls: Type, framework: ModuleType, module_name: str): ...



import typing
from asyncio import AbstractEventLoop
from collections import deque as _deque
from types import ModuleType, TracebackType

import bson
import bson.raw_bson
import pymongo
import pymongo.client_session
import pymongo.collation
import pymongo.command_cursor
import pymongo.database
import pymongo.encryption
import pymongo.read_concern
import pymongo.results
import pymongo.topology_description
import pymongo.typings
from typing_extensions import Self

HAS_SSL: bool
ssl: ModuleType

_Value = typing.TypeVar('_Value')
# _Document = typing.TypeVar('_Document', bound=typing.Mapping[str, typing.Any])
_Type = typing.TypeVar('_Type', bound=typing.Type)
_Cursor = typing.TypeVar('_Cursor', bound=AgnosticBaseCursor)

_Collation = typing.Union[typing.Mapping[str, typing.Any], pymongo.collation.Collation]
_Collection = typing.Union[pymongo.collection.Collection, AgnosticCollection]
_Database = typing.Union[pymongo.database.Database, AgnosticDatabase]
_Document = typing.Mapping[str, typing.Any]
_Pipeline = typing.Sequence[typing.Mapping[str, typing.Any]]
_Session = typing.Union[pymongo.client_session.ClientSession, AgnosticClientSession]
_ReadPreferences = typing.Union[
    pymongo.read_preferences.Primary,
    pymongo.read_preferences.PrimaryPreferred,
    pymongo.read_preferences.Secondary,
    pymongo.read_preferences.SecondaryPreferred,
    pymongo.read_preferences.Nearest,
]
_Operation = typing.Union[
    pymongo.operations.InsertOne,
    pymongo.operations.DeleteOne,
    pymongo.operations.DeleteMany,
    pymongo.operations.ReplaceOne,
    pymongo.operations.UpdateOne,
    pymongo.operations.UpdateMany,
]
_Index = typing.Sequence[
    typing.Tuple[str, typing.Union[int, str, typing.Mapping[str, typing.Any]]]
]
_Key_or_Index = typing.Union[str, _Index]
_IO_Loop = AbstractEventLoop

class AgnosticBase(object):
    def __init__(self, delegate: _Type) -> None: ...

class AgnosticBaseProperties(AgnosticBase):
    codec_options: bson.codec_options.CodecOptions
    read_preference: _ReadPreferences
    read_concern: pymongo.read_concern.ReadConcern
    write_concern: pymongo.write_concern.WriteConcern

class AgnosticClient(AgnosticBaseProperties):
    HOST: str
    PORT: int
    address: typing.Optional[typing.Tuple[str, int]]
    arbiters: typing.Set[typing.Tuple[str, int]]
    is_mongos: bool
    is_primary: bool
    io_loop: _IO_Loop
    nodes: typing.FrozenSet[typing.Tuple[str, typing.Optional[int]]]
    options: pymongo.mongo_client.ClientOptions
    primary: typing.Optional[typing.Tuple[str, int]]
    secondaries: typing.Set[typing.Tuple[str, int]]
    topology_description: pymongo.topology_description.TopologyDescription

    def __getitem__(self, item: str) -> AgnosticDatabase: ...
    def __getattr__(self, item: str) -> AgnosticDatabase: ...
    def __init__(
        self,
        host: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        port: typing.Optional[int] = None,
        document_class: typing.Optional[
            typing.Type[typing.Mapping[str, typing.Any]]
        ] = None,
        tz_aware: typing.Optional[bool] = None,
        connect: typing.Optional[bool] = None,
        type_registry: typing.Optional[bson.codec_options.TypeRegistry] = None,
        **kwargs: typing.Any,
    ) -> None: ...
    def close(self) -> None: ...
    async def drop_database(
        self,
        name_or_database: typing.Union[str, _Database],
        session: typing.Optional[_Session],
        comment: typing.Optional[typing.Any],
    ) -> None: ...
    def get_database(
        self,
        name: typing.Optional[str],
        codec_options: typing.Optional[bson.codec_options.CodecOptions],
        read_preferences: typing.Optional[_ReadPreferences],
        write_concern: typing.Optional[pymongo.write_concern.WriteConcern],
        read_concern: typing.Optional[pymongo.read_concern.ReadConcern],
    ) -> AgnosticDatabase: ...
    def get_default_database(
        self,
        default: typing.Optional[str],
        codec_options: typing.Optional[bson.codec_options.CodecOptions],
        read_preferences: typing.Optional[_ReadPreferences],
        write_concern: typing.Optional[pymongo.write_concern.WriteConcern],
        read_concern: typing.Optional[pymongo.read_concern.ReadConcern],
        comment: typing.Optional[typing.Any],
    ) -> AgnosticDatabase: ...
    def get_io_loop(self) -> _IO_Loop: ...
    async def list_database_names(
        self,
        session: typing.Optional[_Session],
        comment: typing.Optional[typing.Any],
    ) -> list[str]: ...
    async def list_databases(
        self,
        session: typing.Optional[_Session],
        comment: typing.Optional[typing.Any],
        **kwargs: typing.Any,
    ) -> AgnosticCommandCursor: ...
    async def server_info(
        self,
        session: typing.Optional[_Session],
    ) -> typing.Dict[str, typing.Any]: ...
    async def start_session(
        self,
        casual_consistency: typing.Optional[bool],
        default_transaction_options: typing.Optional[
            pymongo.client_session.TransactionOptions
        ] = None,
    ) -> AgnosticClientSession: ...
    def watch(
        self,
        pipeline: typing.Optional[
            typing.Sequence[typing.Mapping[str, typing.Any]]
        ] = None,
        full_document: typing.Optional[str] = None,
        resume_after: typing.Optional[typing.Mapping[str, typing.Any]] = None,
        max_await_time_ms: typing.Optional[int] = None,
        batch_size: typing.Optional[int] = None,
        collation: typing.Optional[
            typing.Union[typing.Mapping[str, typing.Any], pymongo.collation.Collation]
        ] = None,
        start_at_operation_time: typing.Optional[bson.timestamp.Timestamp] = None,
        session: typing.Optional[_Session] = None,
        start_after: typing.Optional[typing.Mapping[str, typing.Any]] = None,
        comment: typing.Optional[typing.Any] = None,
        full_document_before_change: typing.Optional[str] = None,
    ) -> AgnosticChangeStream: ...

class _MotorTransactionContext:
    def __init__(self, session: _Session) -> None: ...
    async def __aenter__(self) -> Self: ...
    async def __aexit__(
        self,
        exc_type: typing.Optional[typing.Type[Exception]],
        exc_val: typing.Optional[Exception],
        exc_tb: typing.Optional[TracebackType],
    ) -> None: ...

class AgnosticClientSession(AgnosticBase):
    client: AgnosticClient
    cluster_time: typing.Optional[typing.Mapping[str, typing.Any]]
    has_ended: bool
    in_transaction: bool
    options: pymongo.client_session.SessionOptions
    operation_time: typing.Optional[bson.timestamp.Timestamp]
    session_id: typing.Mapping[str, typing.Any]

    async def __aenter__(self) -> Self: ...
    async def __aexit__(
        self,
        exc_type: typing.Optional[typing.Type[Exception]],
        exc_val: typing.Optional[Exception],
        exc_tb: typing.Optional[TracebackType],
    ) -> None: ...
    def __enter__(self) -> typing.NoReturn: ...
    def __exit__(
        self,
        exc_type: typing.Optional[typing.Type[Exception]],
        exc_val: typing.Optional[Exception],
        exc_tb: typing.Optional[TracebackType],
    ) -> None: ...
    def advance_cluster_time(
        self, cluster_time: typing.Mapping[str, typing.Any]
    ) -> None: ...
    def advance_operation_time(
        self, operation_time: bson.timestamp.Timestamp
    ) -> None: ...
    async def abort_transaction(self) -> None: ...
    async def commit_transaction(self) -> None: ...
    async def end_session(self) -> None: ...
    def get_io_loop(self) -> _IO_Loop: ...
    def start_transaction(
        self,
        read_concern: typing.Optional[pymongo.read_concern.ReadConcern] = None,
        write_concern: typing.Optional[pymongo.write_concern.WriteConcern] = None,
        read_preference: typing.Optional[_ReadPreferences] = None,
        max_commit_time_ns: typing.Optional[int] = None,
    ) -> _MotorTransactionContext: ...
    async def with_transaction(
        self,
        coro: typing.Callable[[Self], typing.Awaitable[_Value]],
        read_concern: typing.Optional[pymongo.read_concern.ReadConcern] = None,
        write_concern: typing.Optional[pymongo.write_concern.WriteConcern] = None,
        read_preference: typing.Optional[_ReadPreferences] = None,
        max_commit_time_ns: typing.Optional[int] = None,
    ) -> _Value: ...

class AgnosticDatabase(AgnosticBaseProperties):
    name: str
    client: AgnosticClient

    def __bool__(self) -> typing.NoReturn: ...
    def __getitem__(self, item: str) -> AgnosticCollection: ...
    def __getattr__(self, item: str) -> AgnosticCollection: ...
    def __hash__(self) -> int: ...
    def __init__(
        self, client: AgnosticClient, name: str, **kwargs: typing.Any
    ) -> None: ...
    def aggregate(
        self, pipeline: _Pipeline, *args: typing.Any, **kwargs: typing.Any
    ) -> AgnosticLatentCommandCursor: ...
    async def command(
        self,
        command: typing.Union[str, typing.MutableMapping[str, typing.Any]],
        value: typing.Any = 1,
        check: bool = True,
        allowable_errors: typing.Optional[
            typing.Sequence[typing.Union[str, int]]
        ] = None,
        read_preference: typing.Optional[_ReadPreferences] = None,
        codec_options: typing.Optional[
            bson.codec_options.CodecOptions[_Document]
        ] = None,
        session: typing.Optional[_Session] = None,
        comment: typing.Optional[typing.Any] = None,
        **kwargs: typing.Any,
    ) -> _Document: ...
    async def create_collection(
        self,
        name: str,
        codec_options: typing.Optional[bson.codec_options.CodecOptions] = None,
        read_preference: typing.Optional[_ReadPreferences] = None,
        write_concern: typing.Optional[pymongo.write_concern.WriteConcern] = None,
        read_concern: typing.Optional[pymongo.read_concern.ReadConcern] = None,
        session: typing.Optional[_Session] = None,
        check_exists: typing.Optional[bool] = True,
        **kwargs: typing.Any,
    ) -> AgnosticCollection: ...
    async def dereference(
        self,
        dbref: bson.dbref.DBRef,
        session: typing.Optional[_Session],
        comment: typing.Optional[typing.Any],
        **kwargs: typing.Any,
    ) -> _Document: ...
    async def drop_collection(
        self,
        name_or_collection: typing.Union[str, _Collection],
        session: typing.Optional[_Session] = None,
        comment: typing.Optional[typing.Any] = None,
        encrypted_fields: typing.Optional[typing.Mapping[str, typing.Any]] = None,
    ) -> typing.Dict[str, typing.Any]: ...
    def get_collection(
        self,
        name: str,
        codec_options: typing.Optional[bson.codec_options.CodecOptions] = None,
        read_preference: typing.Optional[_ReadPreferences] = None,
        write_concern: typing.Optional[pymongo.write_concern.WriteConcern] = None,
        read_concern: typing.Optional[pymongo.read_concern.ReadConcern] = None,
    ) -> AgnosticCollection: ...
    def get_io_loop(self) -> _IO_Loop: ...
    async def list_collection_names(
        self,
        session: typing.Optional[_Session] = None,
        filter: typing.Optional[typing.Mapping[str, typing.Any]] = None,
        comment: typing.Optional[typing.Any] = None,
        **kwargs: typing.Any,
    ) -> typing.List[str]: ...
    async def list_collections(
        self,
        session: typing.Optional[_Session] = None,
        filter: typing.Optional[typing.Mapping[str, typing.Any]] = None,
        comment: typing.Optional[typing.Any] = None,
        **kwargs: typing.Any,
    ) -> AgnosticCommandCursor: ...
    async def validate_collection(
        self,
        name_or_collection: typing.Union[str, _Collection],
        scandata: bool = False,
        full: bool = False,
        session: typing.Optional[_Session] = None,
        background: typing.Optional[bool] = None,
        comment: typing.Optional[typing.Any] = None,
    ) -> typing.Dict[str, typing.Any]: ...
    def watch(
        self,
        pipeline: typing.Optional[_Pipeline] = None,
        full_document: typing.Optional[str] = None,
        resume_after: typing.Optional[typing.Mapping[str, typing.Any]] = None,
        max_await_time_ms: typing.Optional[int] = None,
        batch_size: typing.Optional[int] = None,
        collation: typing.Optional[_Collation] = None,
        start_at_operation_time: typing.Optional[bson.timestamp.Timestamp] = None,
        session: typing.Optional[_Session] = None,
        start_after: typing.Optional[typing.Mapping[str, typing.Any]] = None,
        comment: typing.Optional[typing.Any] = None,
        full_document_before_change: typing.Optional[str] = None,
    ) -> AgnosticChangeStream: ...
    def with_options(
        self,
        codec_options: typing.Optional[bson.codec_options.CodecOptions] = None,
        read_preference: typing.Optional[_ReadPreferences] = None,
        write_concern: typing.Optional[pymongo.write_concern.WriteConcern] = None,
        read_concern: typing.Optional[pymongo.read_concern.ReadConcern] = None,
    ) -> Self: ...

class AgnosticCollection(AgnosticBaseProperties):
    name: str
    full_name: str

    def __hash__(self) -> int: ...
    def __bool__(self) -> typing.NoReturn: ...
    def __getitem__(self, item: str) -> Self: ...
    def __getattr__(self, item: str) -> Self: ...
    def __init__(
        self,
        database: AgnosticDatabase,
        name: str,
        codec_options: typing.Optional[bson.codec_options.CodecOptions] = None,
        read_preference: typing.Optional[_ReadPreferences] = None,
        write_concern: typing.Optional[pymongo.write_concern.WriteConcern] = None,
        read_concern: typing.Optional[pymongo.read_concern.ReadConcern] = None,
        _delegate: typing.Optional[pymongo.collection.Collection] = None,
    ) -> None: ...
    def aggregate(
        self,
        pipeline: _Pipeline,
        *args: typing.Any,
        **kwargs: typing.Any,
    ) -> AgnosticLatentCommandCursor: ...
    def aggregate_raw_batches(
        self,
        pipeline: _Pipeline,
        **kwargs: typing.Any,
    ) -> AgnosticLatentCommandCursor: ...
    async def bulk_write(
        self,
        requests: typing.Sequence[_Operation],
        ordered: bool = True,
        bypass_document_validation: bool = False,
        session: typing.Optional[_Session] = None,
        comment: typing.Optional[typing.Any] = None,
        let: typing.Optional[typing.Mapping] = None,
    ) -> pymongo.results.BulkWriteResult: ...
    async def count_documents(
        self,
        filter: typing.Mapping[str, typing.Any],
        session: typing.Optional[_Session],
        comment: typing.Optional[typing.Any],
        **kwargs: typing.Any,
    ) -> int: ...
    async def create_index(
        self,
        keys: _Key_or_Index,
        session: typing.Optional[_Session] = None,
        comment: typing.Optional[typing.Any] = None,
        **kwargs: typing.Any,
    ) -> str: ...
    async def create_indexes(
        self,
        indexes: typing.Sequence[pymongo.operations.IndexModel],
        session: typing.Optional[_Session] = None,
        comment: typing.Optional[typing.Any] = None,
        **kwargs: typing.Any,
    ) -> typing.List: ...
    async def delete_many(
        self,
        filter: typing.Mapping[str, typing.Any],
        collation: typing.Optional[_Collation] = None,
        hint: typing.Optional[_Key_or_Index] = None,
        session: typing.Optional[_Session] = None,
        let: typing.Optional[typing.Mapping[str, typing.Any]] = None,
        comment: typing.Optional[typing.Any] = None,
    ) -> pymongo.results.DeleteResult: ...
    async def delete_one(
        self,
        filter: typing.Mapping[str, typing.Any],
        collation: typing.Optional[_Collation] = None,
        hint: typing.Optional[_Key_or_Index] = None,
        session: typing.Optional[_Session] = None,
        let: typing.Optional[typing.Mapping[str, typing.Any]] = None,
        comment: typing.Optional[typing.Any] = None,
    ) -> pymongo.results.DeleteResult: ...
    async def distinct(
        self,
        key: str,
        filter: typing.Optional[typing.Mapping[str, typing.Any]] = None,
        session: typing.Optional[_Session] = None,
        comment: typing.Optional[typing.Any] = None,
        **kwargs: typing.Any,
    ) -> typing.List: ...
    async def drop(
        self,
        session: typing.Optional[_Session] = None,
        comment: typing.Optional[typing.Any] = None,
        encrypted_fields: typing.Optional[typing.Mapping[str, typing.Any]] = None,
    ) -> None: ...
    async def drop_index(
        self,
        index_or_name: _Key_or_Index,
        session: typing.Optional[_Session],
        comment: typing.Optional[typing.Any],
        **kwargs: typing.Any,
    ) -> None: ...
    async def drop_indexes(
        self,
        session: typing.Optional[_Session],
        comment: typing.Optional[typing.Any],
        **kwargs: typing.Any,
    ) -> None: ...
    async def estimated_document_count(
        self,
        comment: typing.Optional[typing.Any],
        **kwargs: typing.Any,
    ) -> int: ...
    def find(
        self,
        filter: typing.Optional[typing.Mapping[str, typing.Any]] = None,
        projection: typing.Optional[
            typing.Mapping[str, typing.Any] | typing.Iterable[str]
        ] = None,
        skip: int = 0,
        limit: int = 0,
        no_cursor_timeout: bool = False,
        cursor_type: int = pymongo.cursor.CursorType.NON_TAILABLE,
        sort: typing.Optional[
            typing.Sequence[tuple[str, int | str | typing.Mapping[str, typing.Any]]]
        ] = None,
        allow_partial_results: bool = False,
        oplog_replay: bool = False,
        batch_size: int = 0,
        collation: typing.Optional[typing.Mapping[str, typing.Any] | _Collation] = None,
        hint: typing.Optional[
            str
            | typing.Sequence[tuple[str, int | str | typing.Mapping[str, typing.Any]]]
        ] = None,
        max_scan: typing.Optional[int] = None,
        max_time_ms: typing.Optional[int] = None,
        max: typing.Optional[
            typing.Sequence[tuple[str, int | str | typing.Mapping[str, typing.Any]]]
        ] = None,
        min: typing.Optional[
            typing.Sequence[tuple[str, int | str | typing.Mapping[str, typing.Any]]]
        ] = None,
        return_key: typing.Optional[bool] = None,
        show_record_id: typing.Optional[bool] = None,
        snapshot: typing.Optional[bool] = None,
        comment: typing.Optional[typing.Any] = None,
        session: typing.Optional[_Session] = None,
        allow_disk_use: typing.Optional[bool] = None,
        let: typing.Optional[bool] = None,
    ) -> AgnosticCursor: ...
    async def find_one(
        self,
        filter: typing.Optional[typing.Mapping[str, typing.Any]] = None,
        projection: typing.Optional[
            typing.Mapping[str, typing.Any] | typing.Iterable[str]
        ] = None,
        skip: int = 0,
        limit: int = 0,  # Ignored by the driver.
        no_cursor_timeout: bool = False,
        cursor_type: int = pymongo.cursor.CursorType.NON_TAILABLE,
        sort: typing.Optional[
            typing.Sequence[tuple[str, int | str | typing.Mapping[str, typing.Any]]]
        ] = None,
        allow_partial_results: bool = False,
        oplog_replay: bool = False,
        batch_size: int = 0,
        collation: typing.Optional[typing.Mapping[str, typing.Any] | _Collation] = None,
        hint: typing.Optional[
            str
            | typing.Sequence[tuple[str, int | str | typing.Mapping[str, typing.Any]]]
        ] = None,
        max_scan: typing.Optional[int] = None,
        max_time_ms: typing.Optional[int] = None,
        max: typing.Optional[
            typing.Sequence[tuple[str, int | str | typing.Mapping[str, typing.Any]]]
        ] = None,
        min: typing.Optional[
            typing.Sequence[tuple[str, int | str | typing.Mapping[str, typing.Any]]]
        ] = None,
        return_key: typing.Optional[bool] = None,
        show_record_id: typing.Optional[bool] = None,
        snapshot: typing.Optional[bool] = None,
        comment: typing.Optional[typing.Any] = None,
        session: typing.Optional[_Session] = None,
        allow_disk_use: typing.Optional[bool] = None,
        let: typing.Optional[bool] = None,
    ) -> typing.Optional[_Document]: ...
    async def find_one_and_delete(
        self,
        filter: typing.Mapping[str, typing.Any],
        projection: typing.Optional[
            typing.Union[typing.Mapping[str, typing.Any], typing.Iterable[str]]
        ] = None,
        sort: typing.Optional[_Index] = None,
        hint: typing.Optional[_Key_or_Index] = None,
        session: typing.Optional[_Session] = None,
        let: typing.Optional[typing.Mapping[str, typing.Any]] = None,
        comment: typing.Optional[typing.Any] = None,
        **kwargs: typing.Any,
    ) -> _Document: ...
    async def find_one_and_replace(
        self,
        filter: typing.Mapping[str, typing.Any],
        replacement: typing.Mapping[str, typing.Any],
        projection: typing.Optional[
            typing.Union[typing.Mapping[str, typing.Any], typing.Iterable[str]]
        ] = None,
        sort: typing.Optional[_Index] = None,
        upsert: bool = False,
        return_document: bool = pymongo.collection.ReturnDocument.BEFORE,
        hint: typing.Optional[_Key_or_Index] = None,
        session: typing.Optional[_Session] = None,
        let: typing.Optional[typing.Mapping[str, typing.Any]] = None,
        comment: typing.Optional[typing.Any] = None,
        **kwargs: typing.Any,
    ) -> _Document: ...
    async def find_one_and_update(
        self,
        filter: typing.Mapping[str, typing.Any],
        update: typing.Union[typing.Mapping[str, typing.Any], _Pipeline],
        projection: typing.Optional[
            typing.Union[typing.Mapping[str, typing.Any], typing.Iterable[str]]
        ] = None,
        sort: typing.Optional[_Index] = None,
        upsert: bool = False,
        return_document: bool = pymongo.collection.ReturnDocument.BEFORE,
        array_filters: typing.Optional[
            typing.Sequence[typing.Mapping[str, typing.Any]]
        ] = None,
        hint: typing.Optional[_Key_or_Index] = None,
        session: typing.Optional[_Session] = None,
        let: typing.Optional[typing.Mapping[str, typing.Any]] = None,
        comment: typing.Optional[typing.Any] = None,
        **kwargs: typing.Any,
    ) -> _Document: ...
    def find_raw_batches(
        self,
        filter: typing.Optional[typing.Mapping[str, typing.Any]] = None,
        projection: typing.Optional[
            typing.Mapping[str, typing.Any] | typing.Iterable[str]
        ] = None,
        skip: int = 0,
        limit: int = 0,
        no_cursor_timeout: bool = False,
        cursor_type: int = pymongo.cursor.CursorType.NON_TAILABLE,
        sort: typing.Optional[
            typing.Sequence[tuple[str, int | str | typing.Mapping[str, typing.Any]]]
        ] = None,
        allow_partial_results: bool = False,
        oplog_replay: bool = False,
        batch_size: int = 0,
        collation: typing.Optional[typing.Mapping[str, typing.Any] | _Collation] = None,
        hint: typing.Optional[
            str
            | typing.Sequence[tuple[str, int | str | typing.Mapping[str, typing.Any]]]
        ] = None,
        max_scan: typing.Optional[int] = None,
        max_time_ms: typing.Optional[int] = None,
        max: typing.Optional[
            typing.Sequence[tuple[str, int | str | typing.Mapping[str, typing.Any]]]
        ] = None,
        min: typing.Optional[
            typing.Sequence[tuple[str, int | str | typing.Mapping[str, typing.Any]]]
        ] = None,
        return_key: typing.Optional[bool] = None,
        show_record_id: typing.Optional[bool] = None,
        snapshot: typing.Optional[bool] = None,
        comment: typing.Optional[typing.Any] = None,
        session: typing.Optional[_Session] = None,
        allow_disk_use: typing.Optional[bool] = None,
        let: typing.Optional[bool] = None,
    ) -> AgnosticRawBatchCursor: ...
    def get_io_loop(self) -> _IO_Loop: ...
    async def index_information(
        self,
        session: typing.Optional[_Session] = None,
        comment: typing.Optional[typing.Any] = None,
    ) -> typing.MutableMapping[str, typing.Any]: ...
    async def insert_many(
        self,
        documents: typing.Iterable[
            typing.Union[_Document, bson.raw_bson.RawBSONDocument]
        ],
        ordered: bool = True,
        bypass_document_validation: bool = False,
        session: typing.Optional[_Session] = None,
        comment: typing.Optional[typing.Any] = None,
    ) -> pymongo.results.InsertManyResult: ...
    async def insert_one(
        self,
        document: typing.Union[_Document, bson.raw_bson.RawBSONDocument],
        bypass_document_validation: bool = False,
        session: typing.Optional[_Session] = None,
        comment: typing.Optional[typing.Any] = None,
    ) -> pymongo.results.InsertOneResult: ...
    def list_indexes(
        self,
        session: typing.Optional[_Session] = None,
        comment: typing.Optional[typing.Any] = None,
    ) -> AgnosticLatentCommandCursor: ...
    async def options(
        self,
        session: typing.Optional[_Session] = None,
        comment: typing.Optional[typing.Any] = None,
    ) -> typing.MutableMapping[str, typing.Any]: ...
    async def rename(
        self,
        new_name: str,
        session: typing.Optional[_Session] = None,
        comment: typing.Optional[typing.Any] = None,
        **kwargs: typing.Any,
    ) -> typing.MutableMapping[str, typing.Any]: ...
    async def replace_one(
        self,
        filter: typing.Mapping[str, typing.Any],
        replacement: typing.Mapping[str, typing.Any],
        upsert: bool = False,
        bypass_document_validation: bool = False,
        collation: typing.Optional[_Collation] = None,
        hint: typing.Optional[_Key_or_Index] = None,
        session: typing.Optional[_Session] = None,
        let: typing.Optional[typing.Mapping[str, typing.Any]] = None,
        comment: typing.Optional[typing.Any] = None,
    ) -> pymongo.results.UpdateResult: ...
    async def update_many(
        self,
        filter: typing.Mapping[str, typing.Any],
        update: typing.Union[typing.Mapping[str, typing.Any], _Pipeline],
        upsert: bool = False,
        array_filters: typing.Optional[
            typing.Sequence[typing.Mapping[str, typing.Any]]
        ] = None,
        bypass_document_validation: typing.Optional[bool] = None,
        collation: typing.Optional[_Collation] = None,
        hint: typing.Optional[_Key_or_Index] = None,
        session: typing.Optional[_Session] = None,
        let: typing.Optional[typing.Mapping[str, typing.Any]] = None,
        comment: typing.Optional[typing.Any] = None,
    ) -> pymongo.results.UpdateResult: ...
    async def update_one(
        self,
        filter: typing.Mapping[str, typing.Any],
        update: typing.Union[typing.Mapping[str, typing.Any], _Pipeline],
        upsert: bool = False,
        bypass_document_validation: bool = False,
        collation: typing.Optional[_Collation] = None,
        array_filters: typing.Optional[
            typing.Sequence[typing.Mapping[str, typing.Any]]
        ] = None,
        hint: typing.Optional[_Key_or_Index] = None,
        session: typing.Optional[_Session] = None,
        let: typing.Optional[typing.Mapping[str, typing.Any]] = None,
        comment: typing.Optional[typing.Any] = None,
    ) -> pymongo.results.UpdateResult: ...
    def watch(
        self,
        pipeline: typing.Optional[_Pipeline] = None,
        full_document: typing.Optional[str] = None,
        resume_after: typing.Optional[typing.Mapping[str, typing.Any]] = None,
        max_await_time_ms: typing.Optional[int] = None,
        batch_size: typing.Optional[int] = None,
        collation: typing.Optional[_Collation] = None,
        start_at_operation_time: typing.Optional[bson.timestamp.Timestamp] = None,
        session: typing.Optional[_Session] = None,
        start_after: typing.Optional[typing.Mapping[str, typing.Any]] = None,
        comment: typing.Optional[typing.Any] = None,
        full_document_before_change: typing.Optional[str] = None,
    ) -> AgnosticChangeStream: ...
    def with_options(
        self,
        codec_options: typing.Optional[bson.codec_options.CodecOptions] = None,
        read_preference: typing.Optional[_ReadPreferences] = None,
        write_concern: typing.Optional[pymongo.write_concern.WriteConcern] = None,
        read_concern: typing.Optional[pymongo.read_concern.ReadConcern] = None,
    ) -> Self: ...

class AgnosticBaseCursor(AgnosticBase):
    address: typing.Optional[typing.Tuple[str, typing.Any]]
    alive: bool
    cursor_id: typing.Optional[int]
    fetch_next: typing.Awaitable[typing.Union[bool, int]]
    session: typing.Optional[AgnosticClientSession]

    def __init__(
        self, cursor: pymongo.cursor.Cursor, collection: AgnosticCollection
    ) -> None: ...
    def __aiter__(self) -> Self: ...
    async def __aenter__(self) -> Self: ...
    async def __anext__(self) -> _Document: ...
    async def __aexit__(
        self,
        exc_type: typing.Optional[typing.Type[Exception]],
        exc_val: typing.Optional[Exception],
        exc_tb: typing.Optional[TracebackType],
    ) -> None: ...
    async def _async_close(self) -> None: ...
    def _buffer_size(self) -> int: ...
    def _data(self) -> typing.NoReturn: ...
    def _killed(self) -> typing.NoReturn: ...
    def _query_flags(self) -> typing.NoReturn: ...
    async def _refresh(self) -> int: ...
    def batch_size(self, batch_size: int) -> Self: ...
    async def close(self) -> None: ...
    def each(
        self,
        callback: typing.Callable[
            [typing.Optional[_Document], typing.Optional[Exception]], typing.Any
        ],
    ) -> None: ...
    def get_io_loop(self) -> _IO_Loop: ...
    async def next(self) -> _Document: ...
    def next_object(self) -> _Document: ...
    async def to_list(self, length: int) -> typing.List[_Document]: ...

class AgnosticCursor(AgnosticBaseCursor):
    def __copy__(self) -> Self: ...
    def __deepcopy__(self, memodict: typing.Optional[typing.Dict] = None) -> Self: ...
    async def _Cursor__die(self, synchronous=False) -> None: ...
    def _data(self) -> _deque: ...
    def _killed(self) -> bool: ...
    def _query_flags(self) -> int: ...
    def allow_disk_use(self, allow_disk_use: bool) -> Self: ...
    def add_option(self, mask: int) -> Self: ...
    def clone(self) -> Self: ...
    def collation(self, collation: _Collation) -> Self: ...
    def comment(self, comment: typing.Any) -> Self: ...
    async def distinct(self, key: str) -> typing.List: ...
    async def explain(self) -> _Document: ...
    def hint(self, index: typing.Optional[_Index]) -> Self: ...
    def limit(self, limit: int) -> Self: ...
    def max(self, spec: _Index) -> Self: ...
    def max_await_time_ms(self, max_await_time_ms: typing.Optional[int]) -> Self: ...
    def max_scan(self, max_scan: typing.Optional[int]) -> Self: ...
    def max_time_ms(self, max_time_ms: typing.Optional[int]) -> Self: ...
    def min(self, spec: _Index) -> Self: ...
    def remove_option(self, mask: int) -> Self: ...
    def rewind(self) -> Self: ...
    def skip(self, skip: int) -> Self: ...
    def sort(
        self,
        key_or_list: _Key_or_Index,
        direction: typing.Optional[typing.Union[int, str]] = None,
    ) -> Self: ...
    def where(self, code: typing.Union[str, bson.code.Code]) -> Self: ...

class AgnosticRawBatchCursor(AgnosticCursor):
    pass

class AgnosticCommandCursor(AgnosticBaseCursor):
    async def _Cursor__die(self, synchronous=False) -> None: ...
    def _data(self) -> _deque: ...
    def _killed(self) -> bool: ...
    def _query_flags(self) -> typing.Literal[0]: ...

class AgnosticRawBatchCommandCursor(AgnosticCommandCursor):
    pass

class AgnosticLatentCommandCursor(AgnosticCommandCursor):
    args: typing.Optional[typing.Tuple]
    kwargs: typing.Optional[typing.Dict]
    start: typing.Optional[
        typing.Callable[[typing.Any], pymongo.command_cursor.CommandCursor[_Document]]
    ]

    def __init__(
        self,
        collection: AgnosticCollection,
        start: typing.Optional[
            typing.Callable[
                [typing.Any], pymongo.command_cursor.CommandCursor[_Document]
            ]
        ],
        *args: typing.Any,
        **kwargs: typing.Any,
    ) -> None: ...

class AgnosticChangeStream(AgnosticBase):
    alive: bool
    resume_token: typing.Optional[typing.Mapping[str, typing.Any]]
    def __init__(
        self,
        target: typing.Union[AgnosticClient, AgnosticDatabase, AgnosticCollection],
        pipeline: typing.Optional[
            typing.Sequence[typing.Mapping[str, typing.Any]]
        ] = None,
        full_document: typing.Optional[str] = None,
        resume_after: typing.Optional[typing.Mapping[str, typing.Any]] = None,
        max_await_time_ms: typing.Optional[int] = None,
        batch_size: typing.Optional[int] = None,
        collation: typing.Optional[
            typing.Union[typing.Mapping[str, typing.Any], pymongo.collation.Collation]
        ] = None,
        start_at_operation_time: typing.Optional[bson.timestamp.Timestamp] = None,
        session: typing.Optional[_Session] = None,
        start_after: typing.Optional[typing.Mapping[str, typing.Any]] = None,
        comment: typing.Optional[typing.Any] = None,
        full_document_before_change: typing.Optional[str] = None,
    ) -> None: ...
    def __aiter__(self) -> Self: ...
    async def __aenter__(self) -> Self: ...
    async def __aexit__(
        self,
        exc_type: typing.Optional[typing.Type[Exception]],
        exc_val: typing.Optional[Exception],
        exc_tb: typing.Optional[TracebackType],
    ) -> None: ...
    async def __anext__(self) -> _Document: ...
    def __enter__(self) -> typing.NoReturn: ...
    def __exit__(
        self,
        exc_type: typing.Optional[typing.Type[Exception]],
        exc_val: typing.Optional[Exception],
        exc_tb: typing.Optional[TracebackType],
    ) -> None: ...
    def _lazy_init(self) -> None: ...
    def _try_next(self) -> typing.Optional[_Document]: ...
    async def _close(self) -> None: ...
    async def close(self) -> None: ...
    def get_io_loop(self) -> _IO_Loop: ...
    def next(self) -> _Document: ...
    def try_next(self) -> typing.Optional[_Document]: ...

class AgnosticClientEncryption(AgnosticBase):
    io_loop: _IO_Loop

    def __init__(
        self,
        kms_providers: typing.Mapping[str, typing.Any],
        key_vault_namespace: str,
        key_vault_client: AgnosticClient,
        codec_options: bson.codec_options.CodecOptions,
        kms_tls_options: typing.Optional[typing.Mapping[str, typing.Any]] = None,
    ) -> None: ...
    async def __aenter__(self) -> Self: ...
    async def __aexit__(
        self,
        exc_type: typing.Optional[typing.Type[Exception]],
        exc_val: typing.Optional[Exception],
        exc_tb: typing.Optional[TracebackType],
    ) -> None: ...
    def __enter__(self) -> typing.NoReturn: ...
    def __exit__(
        self,
        exc_type: typing.Optional[typing.Type[Exception]],
        exc_val: typing.Optional[Exception],
        exc_tb: typing.Optional[TracebackType],
    ) -> None: ...
    async def add_key_alt_name(
        self, id: bson.binary.Binary, key_alt_name: str
    ) -> typing.Any: ...
    async def create_data_key(
        self,
        kms_provider: str,
        master_key: typing.Optional[typing.Mapping[str, typing.Any]] = None,
        key_alt_names: typing.Optional[typing.Sequence[str]] = None,
        key_material: typing.Optional[bytes] = None,
    ) -> bson.binary.Binary: ...
    async def close(self) -> None: ...
    async def delete_key(
        self, id: bson.binary.Binary
    ) -> pymongo.results.DeleteResult: ...
    async def decrypt(self, value: bson.binary.Binary) -> typing.Any: ...
    async def encrypt(
        self,
        value: typing.Any,
        algorithm: str,
        key_id: typing.Optional[bson.binary.Binary] = None,
        key_alt_name: typing.Optional[str] = None,
        query_type: typing.Optional[str] = None,
        contention_factor: typing.Optional[int] = None,
    ) -> bson.binary.Binary: ...
    async def get_key(
        self, id: bson.binary.Binary
    ) -> typing.Optional[bson.raw_bson.RawBSONDocument]: ...
    async def get_keys(self) -> AgnosticCursor: ...
    async def get_key_by_alt_name(
        self, key_alt_name: str
    ) -> typing.Optional[bson.raw_bson.RawBSONDocument]: ...
    async def remove_key_alt_name(
        self, id: bson.binary.Binary, key_alt_name: str
    ) -> typing.Optional[bson.raw_bson.RawBSONDocument]: ...
    async def rewrap_many_data_key(
        self,
        filter: typing.Mapping[str, typing.Any],
        provider: typing.Optional[str] = None,
        master_key: typing.Optional[typing.Mapping[str, typing.Any]] = None,
    ) -> pymongo.encryption.RewrapManyDataKeyResult: ...
    def get_io_loop(self) -> _IO_Loop: ...



# Client
get_database_doc: str
get_default_database_doc: str
start_session_doc: str

# Database
cmd_doc: str
list_collection_names_doc: str

# Collections
bulk_write_doc: str
create_index_doc: str
create_indexes_doc: str
delete_many_doc: str
delete_one_doc: str
drop_doc: str
find_one_doc: str
find_one_and_delete_doc: str
find_one_and_replace_doc: str
find_one_and_update_doc: str
index_information_doc: str
insert_many_doc: str
insert_one_doc: str
mr_doc: str
replace_one_doc: str
update_many_doc: str
update_one_doc: str

# Cursor
cursor_sort_doc: str
where_doc: str

# Client Encryption
create_data_key_doc: str
close_doc: str

# GridFS
gridfs_delete_doc: str
gridfs_download_to_stream_doc: str
gridfs_download_to_stream_by_name_doc: str
gridfs_open_download_stream_doc: str
gridfs_open_download_stream_by_name_doc: str
gridfs_open_upload_stream_doc: str
gridfs_open_upload_stream_with_id_doc: str
gridfs_rename_doc: str
gridfs_upload_from_stream_doc: str
gridfs_upload_from_stream_with_id_doc: str



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['motor-stubs']

package_data = \
{'': ['*'], 'motor-stubs': ['aiohttp/*', 'frameworks/*']}

install_requires = \
['pymongo>=4.3.0', 'typing-extensions>=4.0.0']

extras_require = \
{'motor': ['motor>=3.0.0', 'dnspython>=2.3.0']}

setup_kwargs = {
    'name': 'motor-types',
    'version': '1.0.0b0',
    'description': "Python stubs for Motor, a Non-Blocking MongoDB driver for Python's Tornado and AsyncIO based applications.",
    'long_description': 'Motor-Types\n===========\nPython stubs for [Motor], a Non-Blocking [MongoDB] driver for [Python]\'s [Tornado] and [AsyncIO] based applications.\n\nAbout\n------\nStubs for [Motor] (version 3.0.0+) for substituting the missing type-hints. These stubs are meant to be used along with\npycharm and mypy to facilitate static type-checking. Installing this package adds these `.pyi` files to\n`libs/site-packages/motor`. Currently, only the stubs for [AsyncIO] are supported. You can contribute to stubs for\n[Tornado] by opening a pull request for the same.\n\n**Note:** This project is currently under development and is in no way affiliated with MongoDB. This is an unofficial\nstub package.\n\nHow to use?\n-----------\n\nYou can either install from [PyPI] using [pip] or add files to your project directories manually.\n\n### Installing Using [pip]:\n```commandline\npip install motor-types\n```\n\n### To install [Motor] (and [Dnspython]) alongside the package:\n```commandline\npip install motor-types[motor]\n```\n\n### To add files to the project manually:\nUse this command to clone the repository:\n```commandline\ngit clone "https://github.com/L0RD-ZER0/Motor-Types"\n```\n\nAfterwards, you can do either of the following to use stubs:\n* Copy the stubs manually to either ``libs/site-packages/motor`` or ``libs/site-packages/motor-stubs``, ideally the latter.\n* Add these stubs manually to project directories.\n  * [For MyPy][MyPy-Stubs].\n  * [For PyCharm][PyCharm-Stubs].\n  * For other static type-checking tools, consider referring to their corresponding documentation regarding stubs.\n\nExamples:\n---------\n### Auto-Complete Example\n**Without Stubs:**\n\n![ACNS]\n\n**With Stubs:**\n\n![ACWS]\n\n### Type-Checking Example\n**Without Stubs:**\n\n![TCNS]\n\n**With Stubs:**\n\n![TCWS]\n\nDependencies\n------------\nThis package uses following dependencies:\n* [Poetry] (For Packaging and Publishing)\n* [PyMongo] (For PyMongo related types)\n* [Motor] (For Referencing and for motor installation extra)\n* [Dnspython] (For motor installation extra)\n* [Pre-Commit] (For maintaining code quality)\n* [Typing-Extensions] (For using the latest typing features)\n\nHow to Contribute?\n------------------\nThe simplest contribution you can make is by opening a [GitHub Issue][GH-Issues] or by forking the repository and making\na pull request on the [GitHub Repository][GH-Repo] for the same. The changes can be as simple as improving the\ndocumentation or as big as completing any incomplete section of the typings.\n\n**Note:** All issues and pull-requests are subjected to a preliminary inspection.\n\nLicense\n-------\nThis repository is licensed under MIT License. The [license][License] can be found within the repository.\n\n\n[Motor]: https://github.com/mongodb/motor\n[MongoDB]: https://www.mongodb.com\n[PyMongo]: https://github.com/mongodb/mongo-python-driver\n[Poetry]: https://github.com/python-poetry/poetry\n[pip]: https://pypi.org/project/pip/\n[Dnspython]: https://www.dnspython.org/\n[Pre-Commit]: https://pre-commit.com\n[Typing-Extensions]: https://github.com/python/typing_extensions\n[Python]: https://python.org\n[Tornado]: https://www.tornadoweb.org/\n[Asyncio]: https://docs.python.org/3/library/asyncio.html\n[PyPI]: https://pypi.org/\n[MyPy-Stubs]: https://mypy.readthedocs.io/en/stable/stubs.html#stub-files\n[PyCharm-Stubs]: https://www.jetbrains.com/help/pycharm/stubs.html\n[GH-Repo]: https://github.com/L0RD-ZER0/Motor-Types\n[GH-Issues]: https://github.com/L0RD-ZER0/Motor-Types/issues\n[License]: https://github.com/L0RD-ZER0/Motor-Types/blob/master/LICENSE\n[ACNS]: https://github.com/L0RD-ZER0/Motor-Types/raw/master/examples/auto-complete-example-ns.png\n[ACWS]: https://github.com/L0RD-ZER0/Motor-Types/raw/master/examples/auto-complete-example-ws.png\n[TCNS]: https://github.com/L0RD-ZER0/Motor-Types/raw/master/examples/type-checking-example-ns.png\n[TCWS]: https://github.com/L0RD-ZER0/Motor-Types/raw/master/examples/type-checking-example-ws.png',
    'author': 'L0RD-ZER0',
    'author_email': 'ackerhon@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.8.0,<4.0',
}


setup(**setup_kwargs)
